# Machine Learning Zoomcamp

<a href="https://datatalks.club/courses/2021-winter-ml-zoomcamp.html"><img src="../images/zoomcamp.jpg" /></a>

* [Course overview video](https://www.youtube.com/watch?v=rowoDjPc8HU) and [slides](https://www.slideshare.net/AlexeyGrigorev/ml-zoomcamp-course-overview-and-logistics)
* [Course playlist](https://www.youtube.com/playlist?list=PL3MmuxUbc_hIhxl5Ji8t4O6lPAOpHaCLR)
* [Register for the course here](https://airtable.com/shr6Gz46UZCgJ9l6w) (you can register for the course at any point of time!)
* [Public calendar](https://calendar.google.com/calendar/?cid=cGtjZ2tkbGc1OG9yb2lxa2Vwc2g4YXMzMmNAZ3JvdXAuY2FsZW5kYXIuZ29vZ2xlLmNvbQ) (subscribing works from desktop only)
* Register at [DataTalks.Club](https://DataTalks.Club) and join the `#course-ml-zoomcamp` channel to talk about the course 


## 1. [Introduction to Machine Learning](01-intro/)

- 1.1 [Introduction to Machine Learning](01-intro/01-what-is-ml.md)
- 1.2 [ML vs Rule-Based Systems](01-intro/02-ml-vs-rules.md)
- 1.3 [Supervised Machine Learning](01-intro/03-supervised-ml.md)
- 1.4 [CRISP-DM](01-intro/04-crisp-dm.md)
- 1.5 [Model Selection Process](01-intro/05-model-selection.md)
- 1.6 [Setting up the Environment](01-intro/06-environment.md)
- 1.7 [Introduction to NumPy](01-intro/07-numpy.md)
- 1.8 [Linear Algebra Refresher](01-intro/08-linear-algebra.md)
- 1.9 [Introduction to Pandas](01-intro/09-pandas.md)
- 1.10 [Summary](01-intro/10-summary.md)
- 1.11 [Homework](01-intro/homework.md)

## 2. [Machine Learning for Regression](02-regression/)

- 2.1 [Car price prediction project](02-regression/01-car-price-intro.md)
- 2.2 [Data preparation](02-regression/02-data-preparation.md)
- 2.3 [Exploratory data analysis](02-regression/03-eda.md)
- 2.4 [Setting up the validation framework](02-regression/04-validation-framework.md)
- 2.5 [Linear regression](02-regression/05-linear-regression-simple.md)
- 2.6 [Linear regression: vector form](02-regression/06-linear-regression-vector.md)
- 2.7 [Training linear regression: Normal equation](02-regression/07-linear-regression-training.md)
- 2.8 [Baseline model for car price prediction project](02-regression/08-baseline-model.md)
- 2.9 [Root mean squared error](02-regression/09-rmse.md)
- 2.10 [Using RMSE on validation data](02-regression/10-car-price-validation.md)
- 2.11 [Feature engineering](02-regression/11-feature-engineering.md)
- 2.12 [Categorical variables](02-regression/12-categorical-variables.md)
- 2.13 [Regularization](02-regression/13-regularization.md)
- 2.14 [Tuning the model](02-regression/14-tuning-model.md)
- 2.15 [Using the model](02-regression/15-using-model.md)
- 2.16 [Car price prediction project summary](02-regression/16-summary.md)
- 2.17 [Explore more](02-regression/17-explore-more.md)
- 2.18 [Homework](02-regression/homework.md)


## 3. [Machine Learning for Classification](03-classification/)

- 3.1 [Churn prediction project](03-classification/01-churn-project.md)
- 3.2 [Data preparation](03-classification/02-data-preparation.md)
- 3.3 [Setting up the validation framework](03-classification/03-validation.md)
- 3.4 [EDA](03-classification/04-eda.md)
- 3.5 [Feature importance: Churn rate and risk ratio](03-classification/05-risk.md)
- 3.6 [Feature importance: Mutual information](03-classification/06-mutual-info.md)
- 3.7 [Feature importance: Correlation](03-classification/07-correlation.md)
- 3.8 [One-hot encoding](03-classification/08-ohe.md)
- 3.9 [Logistic regression](03-classification/09-logistic-regression.md)
- 3.10 [Training logistic regression with Scikit-Learn](03-classification/10-training-log-reg.md)
- 3.11 [Model interpretation](03-classification/11-log-reg-interpretation.md)
- 3.12 [Using the model](03-classification/12-using-log-reg.md)
- 3.13 [Summary](03-classification/13-summary.md)
- 3.14 [Explore more](03-classification/14-explore-more.md)
- 3.15 [Homework](03-classification/homework.md)


## 4. [Evaluation Metrics for Classification](04-evaluation/)

- 4.1 [Evaluation metrics: session overview](04-evaluation/01-overview.md)
- 4.2 [Accuracy and dummy model](04-evaluation/02-accuracy.md)
- 4.3 [Confusion table](04-evaluation/03-confusion-table.md)
- 4.4 [Precision and Recall](04-evaluation/04-precision-recall.md)
- 4.5 [ROC Curves](04-evaluation/05-roc.md)
- 4.6 [ROC AUC](04-evaluation/06-auc.md)
- 4.7 [Cross-Validation](04-evaluation/07-cross-validation.md)
- 4.8 [Summary](04-evaluation/08-summary.md)
- 4.9 [Explore more](04-evaluation/09-explore-more.md)
- 4.10 [Homework](04-evaluation/homework.md)


## 5. [Deploying Machine Learning Models](05-deployment/)

- 5.1 [Intro / Session overview](05-deployment/01-intro.md)
- 5.2 [Saving and loading the model](05-deployment/02-pickle.md)
- 5.3 [Web services: introduction to Flask](05-deployment/03-flask-intro.md)
- 5.4 [Serving the churn model with Flask](05-deployment/04-flask-deployment.md)
- 5.5 [Python virtual environment: Pipenv](05-deployment/05-pipenv.md)
- 5.6 [Environment management: Docker](05-deployment/06-docker.md)
- 5.7 [Deployment to the cloud: AWS Elastic Beanstalk (optional)](05-deployment/07-aws-eb.md)
- 5.8 [Summary](05-deployment/08-summary.md)
- 5.9 [Explore more](05-deployment/09-explore-more.md)
- 5.10 [Homework](05-deployment/homework.md)


## 6. [Decision Trees and Ensemble Learning](06-trees/)

- 6.1 [Credit risk scoring project](06-trees/01-credit-risk.md)
- 6.2 [Data cleaning and preparation](06-trees/02-data-prep.md)
- 6.3 [Decision trees](06-trees/03-decision-trees.md)
- 6.4 [Decision tree learning algorithm](06-trees/04-decision-tree-learning.md)
- 6.5 [Decision trees parameter tuning](06-trees/05-decision-tree-tuning.nd)
- 6.6 [Ensemble learning and random forest](06-trees/06-random-forest.md)
- 6.7 [Gradient boosting and XGBoost](06-trees/07-boosting.md)
- 6.8 [XGBoost parameter tuning](06-trees/08-xgb-tuning.md)
- 6.9 [Selecting the best model](06-trees/09-final-model.md)
- 6.10 [Summary](06-trees/10-summary.md)
- 6.11 [Explore more](06-trees/11-explore-more.md)
- 6.12 [Homework](06-trees/homework.md)


## 7. Midterm Project

## 8. Neural Networks and Deep Learning

- Clothes classification project
- TensorFlow and Keras
- loading the images
- etc
- Using a pre-trained model
- CNNs: convolutional layers
- CNNs: dense layers
- Transfer learning
- Creating the clothes classification model
- Keras functional components
- optimizer
- training the model
- Learning Rate
- Model checkpointing
- Adding more layers
- Dropout
- Data augmentation
- Training a larger clothes classification model
- Using the model with Keras
- summary
- explore more
- homework

## 9. Serverless Deep Learning

- intro
- serverless and AWS Lambda
- tensorflow-lite
- converting the model to TF-lite
- preparing images
- using the model in TF-lite
- putting everything together in a Lambda function
- preparing the docker image
  - testing the image locally
- pusting the image to ECR
- creating the lambda function
- creating the API gateway
- summary
- explore more
- homework

## 10. Kubernetes and TensorFlow-Serving

- intro, serving architecture overview
- saved_model format
- tensorflow-serving
  - running TF-serving locally
- communicating with tf-serving from Jupyter
- creating the gateway service
- introduction to Kubernetes
- creating a cluster on AWS (article)
- preparing the images
  - the TF-serving image
  - the gateway image
- deploying to Kubernetes
  - deployment for tf-serving
  - service for tf-serving
- creating the gateway on Kubernetes
  - deploymnet
  - servince - load balancer
- testing it
- deleting the cluster
- summary
- explore more
- homework

## 11. Kubeflow and KFServing

- intro
- installing Kubeflow on AWS
- preparing the model: uploading to S3
- deploying TF models with KF-serving
- accessing the model
- tranformers
- testing it
- deleting the cluster
- summary
- explore more
- homework - no homework

## 12. Capstone Project

## 13. Article

